@@ -1,4 +1,4 @@
-%% Copyright (C) 2014 Colin B. Macdonald
+%% Copyright (C) 2014-2018 Colin B. Macdonald
 %%
 %% This file is part of OctSymPy.
 %%
@@ -17,86 +17,116 @@
 %% If not, see <http://www.gnu.org/licenses/>.
 
 %% -*- texinfo -*-
-%% @deftypefn  {Function File} {@var{f} =} symfun (@var{expr}, @var{vars})
-%% Define a symbolic function (not usually called directly).
+%% @documentencoding UTF-8
+%% @deftypemethod  @@symfun {@var{f} =} symfun (@var{expr}, @var{vars})
+%% Define a symbolic function (not usually invoked directly).
 %%
 %% A symfun can be abstract or concrete.  An abstract symfun
 %% represents an unknown function (for example, in a differential
-%% equation).  A concrete symfun represents a known function such
-%% as f(x) = sin(x).
+%% equation).  A concrete symfun represents a known function such as
+%% @iftex
+%% @math{f(x) = \sin(x)}.
+%% @end iftex
+%% @ifnottex
+%% f(x) = sin(x).
+%% @end ifnottex
 %%
 %% A concrete symfun:
 %% @example
-%% >> syms x
-%% >> f(x) = sin(x)
-%%    @result{} f(x) = (symfun) sin(x)
-%% >> f
-%%    @result{} f(x) = (symfun) sin(x)
-%% >> f(1)
-%%    @result{} ans = (sym) sin(1)
-%% >> f(x)
-%%    @result{} ans = (sym) sin(x)
+%% @group
+%% syms x
+%% f(x) = sin(x)
+%%   @result{} f(x) = (symfun) sin(x)
+%% f
+%%   @result{} f(x) = (symfun) sin(x)
+%% f(1)
+%%   @result{} ans = (sym) sin(1)
+%% f(x)
+%%   @result{} ans = (sym) sin(x)
+%% @end group
 %% @end example
 %%
 %% An abstract symfun:
 %% @example
-%% >> syms g(x)
-%% >> g
-%%    @result{} g(x) = (symfun) g(x)
+%% @group
+%% syms g(x)
+%% g
+%%   @result{} g(x) = (symfun) g(x)
+%% @end group
 %% @end example
-%% and note this creates the sym @code{x} automatically.
+%% (Note this creates the sym @code{x} automatically if it does
+%% not already exist.)
 %%
-%% Alternatively:
+%% Example: multivariable symfuns:
 %% @example
-%% >> x = sym('x');
-%% >> g(x) = sym('g(x)')
-%%    @result{} g(x) = (symfun) g(x)
-%% @end example
-%% Note the following is @strong{not} the way to create an abstract
-%% symfun:
-%% @example
-%% >> g = sym('g(x)')  % just the symbolic expression g(x)
-%%    @result{} g = (sym) g(x)
-%% @end example
-%% Instead, use @code{g(x)} on the left-hand side as above.
-%%
-%% You can make multidimensional concrete or abstract symfuns using
-%% @code{syms g(x,y)}.  However, a bug in the Octave parser in
-%% versions before 4.0 means this 2D example will give a parse
-%% error.  A workaround is to enclose the argument in a string:
-%% @example
-%% >> syms 'g(x, y)'
-%% >> g
-%%    @result{} g(x, y) = (symfun) g(x, y)
+%% @group
+%% syms x y
+%% g(x, y) = 2*x + sin(y)
+%%   @result{} g(x, y) = (symfun) 2⋅x + sin(y)
+%% syms g(x, y)
+%% g
+%%   @result{} g(x, y) = (symfun) g(x, y)
+%% @end group
 %% @end example
 %%
-%% As the above examples demonstrate, it is usually not necessary to
-%% call symfun directly.  However, it can be done:
+%% Example: creating an abstract function formally of two variables
+%% but depending only on @code{x}:
 %% @example
-%% >> syms x y
-%% >> f = symfun(sin(x), x)
-%%    @result{} f(x) = (symfun) sin(x)
-%% >> F = symfun(x*y, [x y])
-%%    @result{} F(x, y) = (symfun) x⋅y
-%% >> g = symfun(sym('g(x)'), x)
-%%    @result{} g(x) = (symfun) g(x)
-%% >> G = symfun(sym('G(x, y)'), [x y])
-%%    @result{} G(x, y) = (symfun) G(x, y)
+%% @group
+%% syms x y h(x)
+%% h(x, y) = h(x)
+%%   @result{} h(x, y) = (symfun) h(x)
+%% @end group
 %% @end example
 %%
-%% This allows, for example, creating an abstract function formally
-%% of x, y but depending only on x:
+%% A symfun can be composed inside another.  For example, to
+%% demonstrate the chain rule in calculus, we might do:
 %% @example
-%% >> syms x y
-%% >> h = symfun(sym('h(x)'), [x y])
-%%    @result{} h(x, y) = (symfun) h(x)
+%% @group
+%% syms f(t) g(t)
+%% F(t) = f(g(t))
+%%   @result{} F(t) = (symfun) f(g(t))
+%% diff(F, t)
+%%   @result{} ans(t) = (symfun)
+%%       d        ⎛ d        ⎞│
+%%       ──(g(t))⋅⎜───(f(ξ₁))⎟│
+%%       dt       ⎝dξ₁       ⎠│ξ₁=g(t)
+%% @end group
+%% @end example
+%%
+%% It is possible to create an abstract symfun without using the
+%% @code{syms} command:
+%% @example
+%% @group
+%% x = sym('x');
+%% g(x) = sym('g(x)')
+%%   @result{} g(x) = (symfun) g(x)
+%% @end group
+%% @end example
+%% (note the @code{x} must be included on the left-hand side.)
+%% However, @code{syms} is safer because this can fail or give
+%% unpredictable results for certain function names:
+%% @example
+%% @group
+%% beta(x) = sym('beta(x)')
+%%   @print{} ??? ... Error ...
+%% @end group
+%% @end example
+%%
+%% It is usually not necessary to call @code{symfun} directly
+%% but it can be done:
+%% @example
+%% @group
+%% f = symfun(x*sin(y), [x y])
+%%   @result{} f(x, y) = (symfun) x⋅sin(y)
+%% g = symfun(sym('g(x)'), x)
+%%   @result{} g(x) = (symfun) g(x)
+%% @end group
 %% @end example
 %%
 %% @seealso{sym, syms}
-%% @end deftypefn
+%% @end deftypemethod
 
-%% Author: Colin B. Macdonald
-%% Keywords: symbolic, symbols, CAS
 
 function f = symfun(expr, vars)
 
@@ -105,7 +135,9 @@ function f = symfun(expr, vars)
     expr = sym(0);
     vars = sym('x');
   elseif (nargin == 1)
-    error('takes two input arguments')
+    print_usage ();
+  elseif (nargin > 2)
+    print_usage ();
   end
 
   % if the vars are in a sym array, put them in a cell array
@@ -117,15 +149,18 @@ function f = symfun(expr, vars)
     end
   end
 
+  % check that vars are unique Symbols
+  cmd = { 'L, = _ins'
+          'if not all([x is not None and x.is_Symbol for x in L]):'
+          '    return False'
+          'return len(set(L)) == len(L)' };
+  if (~ python_cmd (cmd, vars))
+    error('OctSymPy:symfun:argNotUniqSymbols', ...
+          'symfun arguments must be unique symbols')
+  end
+
   if (ischar (expr))
-    % FIXME: drop this later
-    warning('symfun: deprecated: symfun(''f'', x) format not supported')
-    tok = mystrsplit(expr, {'(', ')', ','});
-    fname = strtrim(tok{1});
-    assert (isvarname (fname))
-    cmd = {['_f = sp.Function("' fname '")(*_ins)'] ...
-            'return (_f,)' };
-    expr = python_cmd (cmd, vars{:});
+    error ('symfun(<string>, x) is not supported, see "help symfun" for options')
   end
 
   if (isa(expr, 'symfun'))
@@ -148,6 +183,10 @@ function f = symfun(expr, vars)
 end
 
 
+%!error <Invalid> symfun (1, sym('x'), 3)
+
+%!error <not supported> symfun ('f', sym('x'))
+
 %!test
 %! syms x y
 %! syms f(x)
@@ -209,7 +248,7 @@ end
 %! f(5);
 %! assert (length (argnames (f)) == 1)
 %! assert (isequal (argnames (f), t))
-%! assert (isequal( diff(f,x), sym(0)))
+%! assert (isequal( formula(diff(f,x)), sym(0)))
 
 %!test
 %! % replace g with shorter and specific fcn
@@ -228,26 +267,35 @@ end
 
 %!test
 %! % Bug #41: Octave <= 3.8 parser fails without quotes around 2D fcn
-%! % (put inside eval to hide from 3.6 parser)
-%! if exist('octave_config_info', 'builtin')
-%!   if (compare_versions (OCTAVE_VERSION (), '4.0.0', '>='))
-%!     syms x y
-%!     eval('syms g(x,y)')
-%!     assert (isa (g, 'symfun'))
-%!   end
-%! else  % matlab
-%!   syms x y
-%!   eval('syms g(x,y)')
-%!   assert (isa (g, 'symfun'))
-%! end
+%! syms x y
+%! eval('syms g(x,y)')
+%! assert (isa (g, 'symfun'))
+
+%!test
+%! % and these days it works without eval trick
+%! syms g(x,y)
+%! assert (isa (g, 'symfun'))
 
 %!test
 %! % syms f(x) without defining x
-%! clear
+%! clear x
 %! syms f(x)
 %! assert(isa(f, 'symfun'))
 %! assert(isa(x, 'sym'))
 
+%!test
+%! % SMT compat: symfun indep var overwrites existing var
+%! t = 6;
+%! syms f(t)
+%! assert (logical (t ~= 6))
+
+%!test
+%! % SMT compat: symfun indep var overwrites existing var, even if sym
+%! syms x
+%! t = x;
+%! syms f(t)
+%! assert (~ logical (t == x))
+
 %!test
 %! syms x y
 %! f(x) = x^2;
@@ -260,16 +308,16 @@ end
 
 %!test
 %! % syms has its own parsing code, check it works
-%! syms 'f(x,y)'
+%! syms f(x,y)
 %! g = f;
-%! syms 'f(x, y)'
+%! syms f(x, y)
 %! assert (isequal (f, g))
 %! syms 'f( x,  y  )'
 %! assert (isequal (f, g))
 
 %!test
 %! % syms own parsing code should not reorder the vars
-%! syms 'f(y, x)'
+%! syms f(y, x)
 %! v = f.vars;
 %! assert (isequal (v{1}, y) && isequal (v{2}, x))
 
@@ -294,3 +342,23 @@ end
 %! assert (isa (y, 'symfun'))
 %! y = symfun(x, t);
 %! assert (isa (y, 'symfun'))
+
+%!error <unique symbols>
+%! % Issue #444: invalid args
+%! syms x
+%! f(x, x) = 2*x;
+
+%!error <unique symbols>
+%! % Issue #444: invalid args
+%! syms x y
+%! f(x, y, x) = x + y;
+
+%!error <unique symbols>
+%! % Issue #444: invalid args
+%! syms x y
+%! f(x, y, x) = x + y;
+
+%!error
+%! % Issue #444: expression as arg
+%! syms x
+%! f(2*x) = 4*x;
